## ----eval=FALSE---------------------------------------------------------------
# # Specify the path to the data file (for Run_analyzeResults). The data file will be generated by Run_MCMC.
# 
# data_file <- system.file("extdata", package = "BayesianFitForecast")
# 
# # Specify the path to the option file.
# option_file <- system.file("extdata", "option.R",
#                            package = "BayesianFitForecast")
# 
# # Specify the path to the Excel file including the reported data.
# excel_file <- system.file("extdata", "SanFrancisco.xlsx",
#                           package = "BayesianFitForecast")
# # Specify the path to save the results. By default, it is set to the temporary path.
# output_path <- tempdir()
# 
# # Run the MCMC analysis. It generates the .Rdata file (data_file).
# Run_MCMC(option_file, excel_file, output_path)
# 
# # Analyze the results. It generates all results files within a folder.
# Run_analyzeResults(data_file, option_file, excel_file, output_path)
# 
# # Please note that you can specify any address to data_file, option_file, excel_file, and output_path.

